# frozen_string_literal: true

module ActiveSwagger
  module DSL
    extend ActiveSupport::Concern

    Error = Class.new(StandardError)

    def swagger_attributes
      output = {}
      self.class.swagger_properties.each_pair do |key, value|
        output[key] = send(key.to_sym)
        output[key] = output[key].to_s if value[:type] == :string
      end
      output
    end

    class_methods do
      def swagger_schema_name(name = nil)
        Definitions.add_to_registry(self, :schema_name, name)
      end

      def swagger_attribute(name, options = {})
        Definitions.add_swagger_atribute(self, name, options)
      end

      def swagger_strong_params
        Params.new(swagger_properties).to_strong_params
      end

      def swagger_schema
        { type: :object, properties: swagger_properties }
      end

      def swagger_properties
        class_registry[:schema_attributes]
      end

      def class_registry
        Definitions.registry[to_s]
      end

      def schema_name
        class_registry[:schema_name]
      end

      def swagger_reference
        { '$ref' => "#/components/schemas/#{schema_name}" }
      end
    end
  end
end
